<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\COZ\Services\Pla10500Repository;

class Pla01600 extends \Zeedhi\Framework\Controller\Report {
    protected $entityManager;
    protected $novoCodigo;
    protected $lista;
    protected $nrlistaFilialServico;
    protected $nrlistaFilialSetor;
    protected $pla10500Repository;
    protected $birtStrategy;

    public function __construct(
        ReportService $reportService, $entityManager, $novoCodigo,
        $lista, Pla10500Repository $pla10500Repository, $birtStrategy
    ) {
        $this->entityManager = $entityManager;
        $this->novoCodigo = $novoCodigo;
        $this->lista = $lista;
        $this->pla10500Repository = $pla10500Repository;
        $this->birtStrategy = $birtStrategy;

        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'req01600' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01601' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01602' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req01604' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'TAMNIV' => 'P_TAMNIV',
                        'TIPONIVEL' => 'P_TIPONIVEL',
                        'TAMNIVELPROD' => 'P_TAMNIVELPROD',
                        'ORDENA' => 'P_ORDENA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'NRLISTASETOR' => 'P_CDSETOR',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'NRORG' => 'P_NRORG',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSDIST' => 'P_CONSDIST',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CDPICTPROD' => 'P_CDPICTPROD'
                    )
            ),
        'req00703' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'NRORG' => 'P_NRORG',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'CDPICTPROD' => 'P_CDPICTPROD',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'ORDENA'  => 'P_ORDENA',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        "NMCENTRAL" => "P_NMCENTRAL"
                        // 'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        // 'IDARREDOND' => 'P_ARREDONDA',
                        // 'NRLISTA' => 'P_NRLISTA',
                        // 'CDARVPRODINI' => 'P_PRODINI',
                        // 'CDARVPRODFIN' => 'P_PRODFIN',
                        // 'DSPLANEJPRODUCAO' => 'P_DSPLANEJPRODUCAO',
                    )
            ),
        'req01800' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRLISTASERVICO' => 'P_LISTASERVICO',
                        'NRORG' => 'P_NRORG',
                        'DTINICIAL' => 'P_DTINICIAL',
                        'DTFINAL' => 'P_DTFINAL',
                        'NRPLANPROD' => 'P_NRPLANPROD',
                        'CDCENTRAL' => 'P_CDCENTRAL',
                        'CONSCARD' => 'P_CONSCARD',
                        'CONSFIXO' => 'P_CONSFIXO',
                        'CONSPERCAP' => 'P_CONSPERCAP',
                        'CDARVPRODINI' => 'P_PRODINI',
                        'CDARVPRODFIN' => 'P_PRODFIN',
                        'NRCONSPLAN' => 'P_NRCONSPLAN',
                        'CDOPERADOR' => 'P_CDOPERADOR',
                        'NMOPERADOR' => 'P_NMOPERADOR',
                        'CDPICTPROD' => 'P_CDPICTPROD',
                        'CONSDIST' => 'P_CONSDIST',
                        'IDQUEBRAPAGINA' => 'P_IDQUEBRAPAGINA',
                        'IDARREDOND' => 'P_ARREDONDA',
                        'NRLINHABRANCA' => 'P_NRLINHABRANCA',
                        'ORDENA' => 'P_ORDENA'
                    )
            ),
        'REQ01801' => 
            array (
                'strategy' => 'BIRT',
                'parameterMapping' => 
                    array (
                        'NRPLANPROD'       => 'P_NRPLANPROD',
                        'DSPLANEJPRODUCAO' => 'P_DSPLANPROD',
                        'NRLISTASERVICO'   => 'P_LISTASERVICO',
                        'CDARVPRODINI'     => 'P_PRODINI',
                        'CDARVPRODFIN'     => 'P_PRODFIN',
                        'NRORG'            => 'P_NRORG',
                        'CDCENTRAL'        => 'P_CDCENTRAL',
                        'DTINICIAL'        => 'P_DTINICIAL',
                        'DTFINAL'          => 'P_DTFINAL',
                        'CONSCARD'         => 'P_CONSCARD',
                        'NRCONSPLAN'       => 'P_NRCONSPLAN',
                        'CONSFIXO'         => 'P_CONSFIXO',
                        'CONSPERCAP'       => 'P_CONSPERCAP',
                        'CONSDIST'         => 'P_CONSDIST',
                        'NMCENTRAL'        => 'P_NMCENTRAL',
                        'CDOPERADOR'       => 'P_CDOPERADOR',
                        'NMOPERADOR'       => 'P_NMOPERADOR',
                        'CDPICTPROD'       => 'P_CDPICTPROD',
                        'CDPICTPRAT'       => 'P_CDPICTPRAT',
                        'IDSUPPRAT'        => 'P_IDSUPPRAT'
                    )
            ),
        'req01802' => 
            array (
                'strategy' => 'BIRT',
                'parameterMapping' => 
                    array (
                        'CDOPERADOR'       => 'P_CDOPERADOR',
                        'NMOPERADOR'       => 'P_NMOPERADOR',
                        'NRPLANPROD'       => 'P_NRPLANPROD',
                        'DSPLANEJPRODUCAO' => 'P_DSPLANPROD',
                        'IDARREDOND'       => 'P_ARREDONDA',
                        'CONSCARD'         => 'P_CONSCARD',
                        'NRLISTASERVICO'   => 'P_LISTASERVICO',
                        'NRORG'            => 'P_NRORG',
                        'DTINICIAL'        => 'P_DTINICIAL',
                        'DTFINAL'          => 'P_DTFINAL',
                        'CDCENTRAL'        => 'P_CDCENTRAL',
                        'NRCONSPLAN'       => 'P_NRCONSPLAN',
                        'CONSFIXO'         => 'P_CONSFIXO',
                        'CONSPERCAP'       => 'P_CONSPERCAP',
                        'CONSDIST'         => 'P_CONSDIST',
                        'CDARVPRODINI'     => 'P_PRODINI',
                        'CDARVPRODFIN'     => 'P_PRODFIN',
                        'NRLISTASETOR'     => 'P_CDSETOR',
                        'ORDENA'           => 'P_ORDENA',
                        'CDPICTPROD'       => 'P_CDPICTPROD'
                    )
            )
            
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $row['NRORG'] = $this->pla10500Repository->environment->getNrorgTrab();

        $dtInicial = $row['DTINICIAL'];
        $dtFinal = $row['DTFINAL'];
        
        $consfixo = $row['CONSFIXO'];
        $conspercap = $row['CONSPERCAP'];
        $distribuicao = $row['CONSDIST'];
        
        if (!$row['NRPLANPROD']) {
            $row['NRLISTASERVICO'] = $this->lista->geraListaServico($row['LISTASERVICO'], $row['NRORG']);
            $listaServicos = $row['LISTASERVICO'];
            $listaServicos = array_filter(explode("_", $listaServicos));
            
            $listaServ = array();
            foreach ($listaServicos as $serv) {
                $listaServ[] = array('SERVICO' => $serv);
            }
        } else {
            $connection = $this->entityManager->getConnection();
            $listaServicoFront = $row['LISTASERVICO'];
            $listaServ = null;
            $servicosComuns = [];
            $result = '';

            if ((!empty($row['CDGRUPFILI']) || 
                !empty($row['NMFILIAGREGA']) || 
                !empty($row['NMGRUPSERV']) || 
                !empty($row['NMSERVICO']))) {
                $result = $listaServicoFront;
                $servicosArray = array_filter(explode('_', $listaServicoFront));
            
                $listaServicoFrontArray = array_map(function($servico) {
                    return array("SERVICO" => $servico);
                }, $servicosArray);
            
                $listaServ = $listaServicoFrontArray;
            } else {
                $query = "SELECT DISTINCT CDSERVFILIAGREG, CDFILIAGREG, CDFILIAGREG || CDSERVFILIAGREG AS CONCAT
                            FROM PLANPRODFILIAGREGA
                               WHERE NRPLANPROD = :P_NRPLANPROD";
                $resultado = $connection->fetchAll(
                    $query, array("P_NRPLANPROD" => $row['NRPLANPROD'])
                );
                
                if (!empty($resultado)) {
                    foreach ($resultado as $filiServ) {
                        if (strpos($row['LISTASERVICO'], $filiServ['CONCAT']) >= 0) {
                            $listaServ[] = array('SERVICO' => $filiServ['CONCAT']);
                        } 
                    }
                }
                
                $servicosRow = array_filter(explode('_', $row['LISTASERVICO']));
                foreach ($listaServ as $servico) {
                    if (in_array($servico['SERVICO'], $servicosRow)) {
                        $servicosComuns[] = array('SERVICO' => $servico['SERVICO']);
                        $result = $result.'_'.$servico['SERVICO'];
                    }
                }
                
                $listaServ = $servicosComuns;
            }

            $row['NRLISTASERVICO'] = $this->lista->geraListaServico($result, $row['NRORG']);
        }

        if(isset($listaServ)) {
            if ($row['CD_REPORT_NAME']=== 'REQ01801') $arredondamento = 'N';
            else $arredondamento = $row['IDARREDOND'];
            $consplan = $this->pla10500Repository->consplanGeral(
                array(
                    'NRORG' => $row['NRORG'],
                    'DTCARDAINI' => $dtInicial,
                    'DTCARDAFIN' => $dtFinal,
                    'CDOPERADOR' => $row['CDOPERADOR'],
                    'SERVICOFILIAL' => $listaServ
                ),
                $distribuicao . $consfixo . $conspercap,
                $arredondamento
            );
            
        } else $consplan['P_NRCONSPLAN'] = 'xxxxxxxxxx';
        
        $row['NRCONSPLAN'] = $consplan['P_NRCONSPLAN'];
        $this->pla10500Repository->nrconsplan = $consplan['P_NRCONSPLAN'];
        $row['NRLISTASETOR'] = $this->montaListaFilialServico($row['NRORG'], $row['LISTASETOR']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }

    public function montaListaFilialServico($nrorg, $filiaisServicos){
        $filiaisServicos = rtrim($filiaisServicos, '_');
        $arrayFiliaisServicos = explode('_', $filiaisServicos);
        
        if ($arrayFiliaisServicos[0]) {
            $this->nrlistaFilialServico = $this->novoCodigo->getCodeByTableName(
                "LISTA", $nrorg
            );
            foreach ($arrayFiliaisServicos as $filServ) {
                $FilialServicoParametros = array(
                    "NRORG" => $nrorg,
                    "NRLISTA" => $this->nrlistaFilialServico,
                    "CDLISTA" => $filServ
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '',
                    "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '',
                    "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '',
                    "DTULTATUPRFIL" => ''
                , "__is_new" => true
                );
                $this->saveLista($FilialServicoParametros);
            }
        } else {
            $this->nrlistaFilialServico = 0;
        }

        return $this->nrlistaFilialServico;
    }

    public function montaListaFilialSetor($nrorg, $filiaisServicos){
        $filiaisSetor = rtrim($filiaisServicos, '_');
        $arrayFiliaisSetor = explode('_', $filiaisSetor);
        if ($arrayFiliaisSetor[0]) {
            $this->nrlistaFilialSetor = $this->novoCodigo->getCodeByTableName(
                "LISTA", $nrorg
            );
            foreach ($arrayFiliaisSetor as $setor) {
                $FilialSetorParametros = array(
                    "NRORG" => $nrorg, "NRLISTA" => $this->nrlistaFilialSetor,
                    "CDLISTA" => $setor
                , "DTULTATULIST" => '', "NRORGINCLUSAO" => '',
                    "DTINCLUSAO" => ''
                , "CDOPERINCLUSAO" => '', "IDATIVO" => '', "CDOPERULTATU" => ''
                , "DTULTATU" => '', "NRORGULTATU" => '', "DTULTATUPRFIL" => ''
                , "__is_new" => true
                );
                $this->saveLista($FilialSetorParametros);
            }
        } else {
            $this->nrlistaFilialSetor = 0;
        }
        return $this->nrlistaFilialSetor;
    }

    public function saveLista($param){
        $request = new DTO\Request\Row($param, "save", "/dummy", " ");
        $response = new DTO\Response();
        $this->lista->save($request, $response);
    }
}